/* MSF60 decoding routines
 * The function clock_decode() must be called from the @input()
 * function, which the main script must implement. See the
 * application note for details:
 *    http://www.compuphase.com/mp3/h0420_timecode.htm
 */

#if defined _clock_included
    #endinput
#endif
#define _clock_included

const pulsemargin = 30          /* margin in milliseconds */
const ignoreseconds = 17        /* ignore the first seconds */
const numbits = 60 - ignoreseconds

static msf_pin                  /* the input pin number */
static msf_minutestamp = 0
static msf_timestamp
static msf_pulses[numbits char]
static msf_index = 0

clock_init(power_pin, data_pin)
    {
    configiopin power_pin, Output
    setiopin power_pin, 1

    configiopin data_pin, Input
    msf_pin = data_pin  /* save, for testing it later */

    msf_index = 0
    msf_minutestamp = 0
    }

bool: clock_decode(pin, status)
    {
    new bool: updated = false

    if (pin != msf_pin)
        return false

    if (status)
        {
        msf_timestamp = tickcount()
        return false
        }

    /* check the width of the pulse; according to the specifications, a
     * pulse must be 0.1 or 0.2 seconds
     */
    new pulsewidth = tickcount() - msf_timestamp
    new isverylong = (300 - pulsemargin) <= pulsewidth <= (300 + pulsemargin)
    new islong = (200 - pulsemargin) <= pulsewidth <= (200 + pulsemargin)
    new isshort = (100 - pulsemargin) <= pulsewidth <= (100 + pulsemargin)
    new minutemark = (500 - pulsemargin) <= pulsewidth <= (500 + pulsemargin)
    if (isverylong || islong || isshort)
        {
        /* there are "double pulses" in the first 16 seconds; these are more
         * difficult to handle, so we ignore them
         */
        if (msf_timestamp - msf_minutestamp > (ignoreseconds * 1000) + 500)
            {
            if (msf_index < numbits)
                msf_pulses{msf_index} = 2 * isverylong + islong
            msf_index++
            }
        }
    else if (minutemark)
        {
        new minute, hour, day, month, year
        /* if we got all bits in the array, we can decode the array */
        if (msf_index == numbits
            && msf_pulses{52 - ignoreseconds} == 0
            && msf_pulses{59 - ignoreseconds} == 0)
            {
            /* extract the time */
            minute = msf_pulses{51 - ignoreseconds}
                     + 2 * msf_pulses{50 - ignoreseconds}
                     + 4 * msf_pulses{49 - ignoreseconds}
                     + 8 * msf_pulses{48 - ignoreseconds}
                     + 10 * msf_pulses{47 - ignoreseconds}
                     + 20 * msf_pulses{46 - ignoreseconds}
                     + 40 * msf_pulses{45 - ignoreseconds}
            hour = msf_pulses{44 - ignoreseconds}
                     + 2 * msf_pulses{43 - ignoreseconds}
                     + 4 * msf_pulses{42 - ignoreseconds}
                     + 8 * msf_pulses{41 - ignoreseconds}
                     + 10 * msf_pulses{40 - ignoreseconds}
                     + 20 * msf_pulses{39 - ignoreseconds}
            day = msf_pulses{35 - ignoreseconds}
                     + 2 * msf_pulses{34 - ignoreseconds}
                     + 4 * msf_pulses{33 - ignoreseconds}
                     + 8 * msf_pulses{32 - ignoreseconds}
                     + 10 * msf_pulses{31 - ignoreseconds}
                     + 20 * msf_pulses{30 - ignoreseconds}
            month = msf_pulses{29 - ignoreseconds}
                     + 2 * msf_pulses{28 - ignoreseconds}
                     + 4 * msf_pulses{27 - ignoreseconds}
                     + 8 * msf_pulses{26 - ignoreseconds}
                     + 10 * msf_pulses{25 - ignoreseconds}
            year = msf_pulses{24 - ignoreseconds}
                     + 2 * msf_pulses{23 - ignoreseconds}
                     + 4 * msf_pulses{22 - ignoreseconds}
                     + 8 * msf_pulses{21 - ignoreseconds}
                     + 10 * msf_pulses{20 - ignoreseconds}
                     + 20 * msf_pulses{19 - ignoreseconds}
                     + 40 * msf_pulses{18 - ignoreseconds}
                     + 80 * msf_pulses{17 - ignoreseconds}
                     + 2000

            setdate year, month, day
            settime hour, minute, 0
            updated = true
            }
        msf_minutestamp = msf_timestamp
        msf_index = 0
        }

    return updated
    }
